#!/usr/bin/python

__author__ = "Mohammed Sameer <msameer@foolab.org>"
__version__ = "$Id$"

import sys
import os
import ConfigParser
import syslog
import time
import shutil

try:
    import parser
except Exception, e:
    sys.stderr.write("Can't import the parser module: %s\n" % str(e))
    sys.exit(1)


def __prepare_output(path):
    """
    Make sure a path exists

    It will raise an exception if the path is not there and it failed to create it.

    \param path the path to check.
    """

    if not os.path.isdir(path):
        if os.path.isfile(path):
            os.remove(path)
            os.mkdir(path)
        else:
            os.mkdir(path)

def __read_conf(path, section = "default"):
    """
    \brief Reads the configuration file

    The configuration file is a normal ini-style file. It should have a section called
    "default" so it will be read if this function doesn't get a section passed.
    The keys are db, user, pass, prefix, output, type and host.

    host defaults to localhost
    prefix defaults to an empty string
    type defaults to mysql

    It will raise an exception if it fails to read the file.

    \param path the path to the configuration file.
    \param section the section of the configuration file to read "default" will be used.
    \return a dictionary with the keys (db, user, pass, prefix, output, type, host)
    """

    if not os.path.isfile(path):
        raise Exception, "File %s not found" % path

    conf = {"host": "localhost", "type": "mysql", "prefix": ""}
    __keys = ["user", "pass", "db", "output"]

    cfg = ConfigParser.SafeConfigParser()
    cfg.read(path)

    for key in conf.keys():
        if cfg.has_option(section, key):
            conf[key] = cfg.get(section, key)

    for key in __keys:
        conf[key] = cfg.get(section, key)

    return conf

def __daemonize():
    """daemonize our process"""
    pid = os.fork()
    if pid == 0:
        # don't prevent unmounting....
        os.chdir("/")
        # To become the session leader of this new session and the process group
        # leader of the new process group, we call os.setsid().  The process is
        # also guaranteed not to have a controlling terminal.
        os.setsid()
        os.umask(0)

        # do second fork
        pid = os.fork()
        if pid == 0:
            # Close stdin, stdout and stderr
            null = os.open('/dev/null', os.O_RDWR)
            for i in range(3):
                os.dup2(null, i)
            os.close(null)

        else:
            os._exit(0)
    else:
        # exit() or _exit()?
        # _exit is like exit(), but it doesn't call any functions registered
        # with atexit (and on_exit) or any registered signal handlers.  It also
        # closes any open file descriptors.  Using exit() may cause all stdio
        # streams to be flushed twice and any temporary files may be unexpectedly
        # removed.  It's therefore recommended that child branches of a fork()
        # and the parent branch(es) of a daemon use _exit().
        os._exit(0)# Exit parent of the first child.


def __process(mod, sleep):
    """
    \brief this is the main function. It parses feeds. It never returns.
    \param mod a handle to our database module handler
    \param sleep number of seconds to sleep.
    """
    while True:
        for item in mod.items(conf):
            id = item["id"]
            del item["id"]
            try:
                res = parser.Parser(item)
                res.save()
                file = res.get_file()
                dst = os.path.join(conf["output"], id+".xml")
                shutil.move(file, dst)
                #print dst
            except Exception, e:
                log = "Error parsing item is: %s and URL: %s: %s" % (id, item["url"], e)
                syslog.syslog(syslog.LOG_WARNING, log)
                #        print item
        time.sleep(sleep)


if __name__ == "__main__":
    if len(sys.argv) < 3:
        sys.stderr.write("Usage: %s <path to parser.conf> <seconds to sleep> [section]\n" % sys.argv[0])
        sys.exit(1)

    if len(sys.argv) == 4:
        conf = __read_conf(sys.argv[1], sys.argv[3])
    else:
        conf = __read_conf(sys.argv[1])

    __prepare_output(conf["output"])

    sleep = int(sys.argv[2])

    __daemonize()

    syslog.openlog('fooaggregator daemon', syslog.LOG_WARNING, syslog.LOG_DAEMON)

    #    print conf
    import imp
    fp, pathname, description = imp.find_module(conf["type"])

    try:
        mod = imp.load_module(conf["type"], fp, pathname, description)
    except Exception, e:
        if fp:
            fp.close()
        log = str(e)
        syslog.syslog(syslog.LOG_WARNING, log)
        os.exit(1)

    __process(mod, sleep)
